/**
 * Copyright (c) 2011, Chicken Zombie Bonanza Project
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Chicken Zombie Bonanza Project nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CHICKEN ZOMBIE BONANZA PROJECT BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ucf.chickenzombiebonanza.game.item;

public class HealthInventoryObject extends InventoryObject {
	
	public static final HealthInventoryObject SMALL_HEALTH_KIT = new HealthInventoryObject("Small Health Kit", 10);
	
	public static final HealthInventoryObject MEDIUM_HEALTH_KIT = new HealthInventoryObject("Medium Health Kit", 20);
	
	public static final HealthInventoryObject LARGE_HEALTH_KIT = new HealthInventoryObject("Large Health Kit", 30);
	
	private int count = 1;
	
	private final int healingAmount;
	
	public HealthInventoryObject(String name, int amount) {
		super(name, InventoryObject.InventoryObjectTypeEnum.HEALTH);
		this.healingAmount = amount;
	}
	
	public int getHealingAmount() {
		return healingAmount;
	}
	
	public int getCount() {
		return count;
	}
	
	public void incrementCount(int amount) {
		count += amount;
	}
	
	public boolean use() {
		if(count > 0) {
			count -= 1;
			return true;
		}
		return false;
	}

}
